import boto3
import os

def handler(event, context):

    bucket_name = os.environ['BUCKET']
    region = os.environ['REGION']

    return {
        'statusCode': 200,
        'body': f'Hello World with bucket {bucket_name} on region {region}!'
    }


import os
import shutil
from datetime import datetime
import rony

def copy_files(LOCAL_PATH, project_name):
    shutil.copy(os.path.join(rony.__path__[0], 'base_files', 'etl', 'Dockerfile'), os.path.join(LOCAL_PATH, project_name, 'etl'))
    shutil.copy(os.path.join(rony.__path__[0], 'base_files', 'etl', 'run.py'), os.path.join(LOCAL_PATH, project_name, 'etl'))
    shutil.copy(os.path.join(rony.__path__[0], 'base_files', 'etl', 'lambda_function.py'), os.path.join(LOCAL_PATH, project_name, 'etl'))
    shutil.copy(os.path.join(rony.__path__[0], 'base_files', 'etl', 'lambda_requirements.txt'), os.path.join(LOCAL_PATH, project_name, 'etl'))
    
    shutil.copy(os.path.join(rony.__path__[0], 'base_files', 'requirements.txt'), os.path.join(LOCAL_PATH, project_name))
    shutil.copy(os.path.join(rony.__path__[0], 'base_files', '.gitignore'), os.path.join(LOCAL_PATH, project_name))

    shutil.copy(os.path.join(rony.__path__[0], 'base_files', 'infrastructure', 'cloudwatch.tf'), os.path.join(LOCAL_PATH, project_name, 'infrastructure'))
    shutil.copy(os.path.join(rony.__path__[0], 'base_files', 'infrastructure', 'glue_crawler.tf'), os.path.join(LOCAL_PATH, project_name, 'infrastructure'))
    shutil.copy(os.path.join(rony.__path__[0], 'base_files', 'infrastructure', 'iam.tf'), os.path.join(LOCAL_PATH, project_name, 'infrastructure'))
    shutil.copy(os.path.join(rony.__path__[0], 'base_files', 'infrastructure', 'lambda.tf'), os.path.join(LOCAL_PATH, project_name, 'infrastructure'))
    shutil.copy(os.path.join(rony.__path__[0], 'base_files', 'infrastructure', 'main.tf'), os.path.join(LOCAL_PATH, project_name, 'infrastructure'))
    shutil.copy(os.path.join(rony.__path__[0], 'base_files', 'infrastructure', 's3.tf'), os.path.join(LOCAL_PATH, project_name, 'infrastructure'))
    shutil.copy(os.path.join(rony.__path__[0], 'base_files', 'infrastructure', 'variables.tf'), os.path.join(LOCAL_PATH, project_name, 'infrastructure'))
    shutil.copy(os.path.join(rony.__path__[0], 'base_files', 'infrastructure', 'ecr.tf'), os.path.join(LOCAL_PATH, project_name, 'infrastructure'))

    shutil.copy(os.path.join(rony.__path__[0], 'base_files', 'scripts', 'build_lambda_package.sh'), os.path.join(LOCAL_PATH, project_name, 'scripts'))

    shutil.copy(os.path.join(rony.__path__[0], 'base_files', 'tests', 'test_lambda.py'), os.path.join(LOCAL_PATH, project_name, 'tests'))

    shutil.copy(os.path.join(rony.__path__[0], 'base_files', 'ci', 'github_ci.yml'), os.path.join(LOCAL_PATH, project_name, '.github', 'workflows'))
    shutil.copy(os.path.join(rony.__path__[0], 'base_files', 'ci', 'README.md'), os.path.join(LOCAL_PATH, project_name, '.github', 'workflows'))

    shutil.copy(os.path.join(rony.__path__[0], 'base_files', 'dags', 'conditional_example.py'), os.path.join(LOCAL_PATH, project_name, 'dags'))
    shutil.copy(os.path.join(rony.__path__[0], 'base_files', 'dags', 'titanic_example.py'), os.path.join(LOCAL_PATH, project_name, 'dags'))


def write_readme_file(LOCAL_PATH, project_name):
    with open(os.path.join(LOCAL_PATH, project_name, 'README.md'), 'w+') as outfile:
        outfile.write(f"""# {project_name}

Project started in {datetime.today().strftime("%B %d, %Y")}.

**Please, complete here information on using and testing this project.**
""")


import click
import os
import re
import sys
from .writer import copy_files, write_readme_file
from .validation import get_operational_system, check_version_python, check_python_compile
from .__init__ import __version__ as version

LOCAL_PATH = os.getcwd()

logo = r"""

 _ __ ___  _ __  _   _ 
| '__/ _ \| '_ \| | | |
| | | (_) | | | | |_| |
|_|  \___/|_| |_|\__, |
                 |___/ 
v{}
""".format(version)

@click.group()
def cli():
    pass

@cli.command()
def info():
    """
    Checks that Rony is correctly installed
    """
    click.echo(logo)


@cli.command()
@click.argument('project_name')
def new(project_name):
    """
    Create a new Rony project
    """
    click.echo(f"Creating project {project_name}")
    # Create project folders
    os.makedirs(os.path.join(LOCAL_PATH, project_name, 'etl/notebooks'))
    os.makedirs(os.path.join(LOCAL_PATH, project_name, 'dags'))
    os.makedirs(os.path.join(LOCAL_PATH, project_name, 'scripts'))
    os.makedirs(os.path.join(LOCAL_PATH, project_name, 'infrastructure'))
    os.makedirs(os.path.join(LOCAL_PATH, project_name, 'tests'))
    os.makedirs(os.path.join(LOCAL_PATH, project_name, '.github/workflows'))

    # Copy project files
    copy_files(LOCAL_PATH, project_name)
    write_readme_file(LOCAL_PATH, project_name)

    os.chdir(project_name)
    env_name = f"{project_name}_env"
    os.environ["ENV_NAME"] = env_name
    check_python_compile()

    # Create git repo
    os.system('git init')
    print("A git repository was created. You should add your files and make your first commit.\n")


@click.argument('image_name')
@cli.command()
def build(image_name):
    """
    Build a docker image with given image_name. Only run if you have docker installed.
    One should be at the root directory.
    """
    if not os.path.exists('Dockerfile'):
        click.echo("You gotta have a Dockerfile file.")
    else:
        os.system(f'docker build -t {image_name} .')


@click.argument('image_name')
@cli.command()
def run(image_name):
    """
    Run a container with given image_name. 
    Only run if you have docker installed.
    """
    if not os.path.exists('Dockerfile'):
        click.echo("You gotta have a Dockerfile file")
    else:
        os.system(f'docker run --rm {image_name}')



from airflow import DAG
from airflow.operators.bash_operator import BashOperator
from airflow.operators.python_operator import PythonOperator
from datetime import datetime, timedelta
import pandas as pd

default_args = {
    'owner': 'A3Data',
    "depends_on_past": False,
    "start_date": datetime(2020, 11, 14, 23, 50),
    "email": ["airflow@airflow.com"],
    "email_on_failure": False,
    "email_on_retry": False,
    "retries": 1,
    "retry_delay": timedelta(minutes=1),
}

dag = DAG(
    "titanic-example", 
    description="Get Titanic data from internet and calculate mean age",
    default_args=default_args, 
    schedule_interval=timedelta(minutes=5)
)

get_data = BashOperator(
    task_id="get-data",
    bash_command='curl https://raw.githubusercontent.com/A3Data/hermione/master/hermione/file_text/train.csv -o ~/train.csv',
    dag=dag
)

def calculate_mean_age():
    df = pd.read_csv('~/train.csv')
    med = df.Age.mean()
    return med

task_calculate_mean = PythonOperator(
    task_id='calculate-mean-age',
    python_callable=calculate_mean_age,
    dag=dag
)

def print_age(**context):
    value = context['task_instance'].xcom_pull(task_ids='calculate-mean-age')
    print(f"Mean age is {value}")

task_mean_age = PythonOperator(
    task_id="say-mean-age",
    python_callable=print_age,
    provide_context=True,
    dag=dag
)

get_data >> task_calculate_mean >> task_mean_age


from click.testing import CliRunner
from rony.cli import cli, logo
import os

def test_installation_ok():
    runner = CliRunner()
    res = runner.invoke(cli)
    assert res.exit_code == 0

def test_info():
    runner = CliRunner()
    res = runner.invoke(cli, ['info'])
    assert logo in res.output



from airflow import DAG
from airflow.operators.bash_operator import BashOperator
from airflow.operators.python_operator import PythonOperator, BranchPythonOperator
from datetime import datetime, timedelta
import zipfile
import random
import pandas as pd

default_args = {
    'owner': 'A3Data',
    "depends_on_past": False,
    "start_date": datetime(2020, 12, 30, 18, 10),
    "email": ["airflow@airflow.com"],
    "email_on_failure": False,
    "email_on_retry": False
    #"retries": 1,
    #"retry_delay": timedelta(minutes=1),
}

dag = DAG(
    "conditional-example", 
    description="A DAG with a condition",
    default_args=default_args, 
    schedule_interval=timedelta(minutes=2)
)

get_data = BashOperator(
    task_id="get-data",
    bash_command='curl https://download.inep.gov.br/microdados/Enade_Microdados/microdados_enade_2019.zip -o microdados_enade_2019.zip',
    trigger_rule="all_done",
    dag=dag
)


def unzip_file():
    with zipfile.ZipFile("microdados_enade_2019.zip", 'r') as zipped:
        zipped.extractall()

unzip_data = PythonOperator(
    task_id='unzip-data',
    python_callable=unzip_file,
    dag=dag
)


def select_student():
    df = pd.read_csv('microdados_enade_2019/2019/3.DADOS/microdados_enade_2019.txt', sep=';', decimal=',')
    choice = random.randint(0, df.shape[0]-1)
    student = df.iloc[choice]
    return student.TP_SEXO

pick_student = PythonOperator(
    task_id="pick-student",
    python_callable=select_student,
    dag=dag
)

def MorF(**context):
    value = context['task_instance'].xcom_pull(task_ids='pick-student')
    if value == 'M':
        return 'male_branch'
    elif value == 'F':
        return 'female_branch'

male_or_female = BranchPythonOperator(
    task_id='condition-male_or_female',
    python_callable=MorF,
    provide_context=True,
    dag=dag
)


male_branch = BashOperator(
    task_id="male_branch",
    bash_command='echo "A male student was randomly picked."',
    dag=dag
)

female_branch = BashOperator(
    task_id="female_branch",
    bash_command='echo "A female student was randomly picked."',
    dag=dag
)

get_data >> unzip_data >> pick_student >> male_or_female >> [male_branch, female_branch]


import os
import sys
from os import path

def get_operational_system():
    """
    Check which user's operating system    
    """
    os = sys.platform
    if sys.platform == "linux":
        return "Linux"
    elif sys.platform == "win32":
        return "Windows"
    elif sys.platform == "darwin":
        return "MacOS"
    elif sys.platform == "freebsd8":
        return "FreeBSD"


def check_version_python():
    """
    Checks whether the version of python installed on the system is compatible
    """
    if sys.version_info >= (3,6):
        return True
    else:
        return False

def check_python_compile():
    """
    Checks python3 path location 
    """
    env_name = os.environ.get("ENV_NAME")
    message_env = f"Creating virtual environment {env_name}"
    if get_operational_system() == "Linux" and check_version_python() == True:
        if path.exists("/usr/bin/python3") == True:
            print(message_env)
            os.system(f"python3 -m venv {env_name}")
        else:
            print(message_env)
            os.system(f"python -m venv {env_name}")
    elif get_operational_system() == "Windows" and check_version_python() == True:
        print(message_env)
        os.system(f"python -m venv {env_name}")
    elif get_operational_system() == "MacOS" and check_version_python() == True:
        print(message_env)
        os.system(f"python -m venv {env_name}")
    elif check_version_python == False:
        print('The python version is not supported. Rony is compatible with the version of >= Python 3.6')


from setuptools import setup, find_packages
import re
import os

exec(open('rony/_version.py').read())

with open('README.md', encoding='utf-8') as f:
    long_description = f.read()

setup(
    name='rony',
    version=__version__,
    author='A3Data',
    author_email='rony@a3data.com.br',
    url='https://github.com/A3Data/rony',
    description='An opinionated Data Engineering framework',
    long_description=long_description,
    long_description_content_type='text/markdown',
    packages=find_packages(),
    include_package_data=True,
    classifiers=[
        'Development Status :: 4 - Beta',
        'License :: OSI Approved :: Apache Software License',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Topic :: Software Development'
      ],
      keywords='data engineering mlops devops pipelines',
      license='Apache License 2.0',
    install_requires=[
        'Click'
    ],
    entry_points='''
        [console_scripts]
        rony=rony.cli:cli
    ''',
    python_requires='>=3.6'
)



# Example os simple processing job done in sagemaker
import argparse
import pandas as pd

def get_data():
    return pd.read_csv('https://raw.githubusercontent.com/A3Data/hermione/master/hermione/file_text/train.csv')

def make_transformation(df):
    """
    Calculate mean centered age
    """
    df['mcAge'] = df.Age - df.Age.mean()
    return df

def filter_sex(sex):
    return df.loc[df.Sex == sex]

if __name__ == "__main__":
    
    parser = argparse.ArgumentParser()
    parser.add_argument("-s", "--sex", help="choose a sex category to filter data", 
                        type=str, choices=['male', 'female', 'both'], default='both')
    args = parser.parse_args()
    
    df = get_data()

    if args.sex != 'both':
        df = filter_sex(args.sex)
    
    df = make_transformation(df)
    
    print(df)